#include "../../src/Library/Evolutions/EvolutionData.hpp"
#include "../Core.hpp"
using namespace PkmnLib::Library;

export const EvolutionData* PkmnLib_EvolutionData_CreateLevelEvolution(uint8_t level, const PokemonSpecies* into) {
    return EvolutionData::CreateLevelEvolution(level, into);
}
export const EvolutionData* PkmnLib_EvolutionData_CreateFriendshipEvolution(uint8_t friendship,
                                                                            const PokemonSpecies* into) {
    return EvolutionData::CreateFriendshipEvolution(friendship, into);
}
export const EvolutionData* PkmnLib_EvolutionData_CreateKnownMoveEvolution(const MoveData* move,
                                                                           const PokemonSpecies* into) {
    return EvolutionData::CreateKnownMoveEvolution(move, into);
}
export const EvolutionData* PkmnLib_EvolutionData_CreateLocationEvolution(const char* location,
                                                                          const PokemonSpecies* into) {
    return EvolutionData::CreateLocationEvolution(Arbutils::CaseInsensitiveConstString(location), into);
}
export const EvolutionData* PkmnLib_Evolution_CreateTimeEvolution(TimeOfDay time, const PokemonSpecies* into) {
    return EvolutionData::CreateTimeEvolution(time, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateItemEvolution(const Item* item, const PokemonSpecies* into) {
    return EvolutionData::CreateHeldItemEvolution(item, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateGenderBasedEvolution(CreatureLib::Library::Gender gender,
                                                                         uint8_t level, const PokemonSpecies* into) {
    return EvolutionData::CreateGenderBasedEvolution(gender, level, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateItemUseEvolution(const Item* item, const PokemonSpecies* into) {
    return EvolutionData::CreateItemUseEvolution(item, into);
}

export const EvolutionData* PkmnLib_Evolution_CreateItemUseWithGenderEvolution(const Item* item,
                                                                               CreatureLib::Library::Gender gender,
                                                                               const PokemonSpecies* into) {
    return EvolutionData::CreateItemUseWithGenderEvolution(item, gender, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateTradeEvolution(const PokemonSpecies* into) {
    return EvolutionData::CreateTradeEvolution(into);
}
export const EvolutionData* PkmnLib_Evolution_CreateTradeWithItemEvolution(const Item* item,
                                                                           const PokemonSpecies* into) {
    return EvolutionData::CreateTradeWithItemEvolution(item, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateTradeWithSpeciesEvolution(const PokemonSpecies* traded,
                                                                              const PokemonSpecies* into) {
    return EvolutionData::CreateTradeWithSpeciesEvolution(traded, into);
}
export const EvolutionData* PkmnLib_Evolution_CreateCustomEvolution(const CreatureLib::Library::EffectParameter** data,
                                                                    size_t dataLength, const PokemonSpecies* into) {
    auto list = Arbutils::Collections::List<const CreatureLib::Library::EffectParameter*>(data, data + dataLength);
    return EvolutionData::CreateCustomEvolution(list, into);
}

export EvolutionMethod PkmnLib_Evolution_GetMethod(const EvolutionData* data) { return data->GetMethod(); }
export const PokemonSpecies* PkmnLib_Evolution_GetNewSpecies(const EvolutionData* data) {
    return data->GetNewSpecies();
}
export size_t PkmnLib_Evolution_GetDataCount(const EvolutionData* data) { return data->GetDataCount(); }
export uint8_t PkmnLib_Evolution_GetData(const EvolutionData* data, size_t index,
                                         const CreatureLib::Library::EffectParameter*& out) {
    Try(out = data->GetData(index));
}
