#ifndef PKMNLIB_CREATEPOKEMON_HPP
#define PKMNLIB_CREATEPOKEMON_HPP

#include <utility>
#include "Pokemon.hpp"
namespace PkmnLib::Battling {
    class CreatePokemon {
    private:
        const BattleLibrary* _library;
        Arbutils::CaseInsensitiveConstString _species = ""_cnc;
        Arbutils::CaseInsensitiveConstString _forme = "default"_cnc;
        uint8_t _level;
        std::string _nickname = "";

        Arbutils::CaseInsensitiveConstString _ability = ""_cnc;
        Arbutils::CaseInsensitiveConstString _nature;
        CreatureLib::Library::Gender _gender = static_cast<CreatureLib::Library::Gender>(-1);
        Arbutils::CaseInsensitiveConstString _heldItem = ""_cnc;
        uint32_t _identifier = 0;
        std::vector<std::tuple<const Library::MoveData*, CreatureLib::Battling::AttackLearnMethod>> _attacks = {};

        uint8_t _ivHp = 0;
        uint8_t _ivAttack = 0;
        uint8_t _ivDefense = 0;
        uint8_t _ivSpAtt = 0;
        uint8_t _ivSpDef = 0;
        uint8_t _ivSpeed = 0;

        uint8_t _evHp = 0;
        uint8_t _evAttack = 0;
        uint8_t _evDefense = 0;
        uint8_t _evSpAtt = 0;
        uint8_t _evSpDef = 0;
        uint8_t _evSpeed = 0;

        bool _shininessSet = false;
        bool _isShiny = false;

    public:
        CreatePokemon(const BattleLibrary* library, const Arbutils::CaseInsensitiveConstString& species, uint8_t level)
            : _library(library), _species(species), _level(level) {}

        CreatePokemon* WithForme(const Arbutils::CaseInsensitiveConstString& forme);
        CreatePokemon* WithGender(CreatureLib::Library::Gender gender);
        CreatePokemon* IsShiny(bool value);
        CreatePokemon* WithHeldItem(const Arbutils::CaseInsensitiveConstString& item);
        CreatePokemon* LearnMove(const Arbutils::CaseInsensitiveConstString& move, CreatureLib::Battling::AttackLearnMethod method);

        CreatePokemon* WithRandomIndividualValues(Arbutils::Random rand = Arbutils::Random());
        CreatePokemon* WithIndividualValue(CreatureLib::Library::Statistic stat, uint8_t value);
        CreatePokemon* WithIndividualValues(uint8_t hp, uint8_t att, uint8_t def, uint8_t spAtt, uint8_t spDef,
                                            uint8_t speed);
        CreatePokemon* WithEffortValue(CreatureLib::Library::Statistic stat, uint8_t value);
        CreatePokemon* WithEffortValues(uint8_t hp, uint8_t att, uint8_t def, uint8_t spAtt, uint8_t spDef,
                                            uint8_t speed);

        CreatePokemon* WithNature(const Arbutils::CaseInsensitiveConstString& nature);

        Pokemon* Build();
    };
}

#endif // PKMNLIB_CREATEPOKEMON_HPP
