#ifndef PKMNLIB_NATURELIBRARY_HPP
#define PKMNLIB_NATURELIBRARY_HPP

#include <Arbutils/Collections/Dictionary.hpp>
#include <Arbutils/Collections/List.hpp>
#include <Arbutils/ConstString.hpp>
#include <Arbutils/Random.hpp>
#include <CreatureLib/Library/Exceptions/CreatureException.hpp>
#include <unordered_map>
#include <vector>
#include "Nature.hpp"
namespace PkmnLib::Library {
    class NatureLibrary {
    private:
        Arbutils::Collections::Dictionary<Arbutils::CaseInsensitiveConstString, const Nature*> _items;

    public:
        explicit NatureLibrary(size_t size = 32) noexcept
            : _items(Arbutils::Collections::Dictionary<Arbutils::CaseInsensitiveConstString, const Nature*>(size)) {}

        ~NatureLibrary() {
            for (auto& kv : _items) {
                delete kv.second;
            }
        }

        inline void LoadNature(const Arbutils::CaseInsensitiveConstString& name, const Nature* nature) {
            _items.Insert(name, nature);
        }

        inline const Nature* GetNatureByName(const Arbutils::CaseInsensitiveConstString& name) const {
            return _items[name];
        }

        inline const Arbutils::CaseInsensitiveConstString&
        GetRandomNatureName(Arbutils::Random rand = Arbutils::Random()) const {
            auto i = rand.Get(_items.Count());
            auto& map = _items.GetStdMap();
            return std::next(std::begin(map), i)->first;
        }
        inline const Arbutils::CaseInsensitiveConstString&
        GetRandomNatureName(Arbutils::Random* rand) const {
            auto i = rand->Get(_items.Count());
            auto& map = _items.GetStdMap();
            return std::next(std::begin(map), i)->first;
        }

        inline Arbutils::CaseInsensitiveConstString GetNatureName(const Nature* nature) {
            for (const auto& v : _items) {
                if (v.second == nature) {
                    return v.first;
                }
            }
            throw CreatureException("Nature not found.");
        }
    };
}

#endif // PKMNLIB_NATURELIBRARY_HPP
