#ifndef PKMNLIB_ANGELSCRIPRESOLVER_HPP
#define PKMNLIB_ANGELSCRIPRESOLVER_HPP

#include <CreatureLib/Battling/ScriptHandling/ScriptResolver.hpp>
#include "../../../extern/angelscript_addons/scriptbuilder/scriptbuilder.h"
#include "../../Battling/Library/BattleLibrary.hpp"

#define ANGELSCRIPT_DLL_LIBRARY_IMPORT
#include <angelscript.h>
#include <iostream>
#include "AngelScriptScript.hpp"
#include "AngelScriptTypeInfo.hpp"
using namespace Arbutils::Collections;

class AngelScripResolver : public CreatureLib::Battling::ScriptResolver {
private:
    asIScriptEngine* _engine = nullptr;
    asIScriptModule* _mainModule = nullptr;
    ContextPool* _contextPool = nullptr;
    CScriptBuilder _builder;

    static void MessageCallback(const asSMessageInfo* msg, void* param);
    static void Print(const std::string& str) { std::cout << str << std::endl; }
    Dictionary<ScriptCategory, Dictionary<ConstString, AngelScriptTypeInfo*>> _typeDatabase;

    void RegisterTypes();
    void InitializeByteCode(asIBinaryStream* stream,
                      const Dictionary<ScriptCategory, Dictionary<ConstString, const char*>>& types);

public:
    ~AngelScripResolver() override {
        delete _contextPool;
        for (const auto& category : _typeDatabase) {
            for (const auto& type : category.second) {
                delete type.second;
            }
        }
        _engine->ShutDownAndRelease();
    }

    void Initialize(CreatureLib::Battling::BattleLibrary* library) override;
    void CreateScript(const char* name, const char* script);

    void FinalizeModule();

    CreatureLib::Battling::Script* LoadScript(ScriptCategory category, const ConstString& scriptName) override;

    void WriteByteCodeToFile(const char* file, bool stripDebugInfo = false);
    void LoadByteCodeFromFile(const char* file,
                              const Dictionary<ScriptCategory, Dictionary<ConstString, const char*>>& types);
    uint8_t* WriteByteCodeToMemory(size_t& size, bool stripDebugInfo = false);
    void LoadByteCodeFromMemory(uint8_t*, size_t size,
                                const Dictionary<ScriptCategory, Dictionary<ConstString, const char*>>& types);

    const Dictionary<ScriptCategory, Dictionary<ConstString, AngelScriptTypeInfo*>>& GetTypeDatabase() const noexcept {
        return _typeDatabase;
    }
};
#endif // PKMNLIB_ANGELSCRIPRESOLVER_HPP
