#include "RegisterTurnChoices.hpp"
#include <CreatureLib/Battling/TurnChoices/AttackTurnChoice.hpp>
#include <CreatureLib/Battling/TurnChoices/BaseTurnChoice.hpp>
#include <cassert>
#include "../RefCast.hpp"

void RegisterTurnChoices::Register(asIScriptEngine* engine) {
    RegisterTurnChoiceKindEnum(engine);
    RegisterBaseTurnChoice(engine);
    RegisterMoveTurnChoice(engine);
}
void RegisterTurnChoices::RegisterTurnChoiceKindEnum(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterEnum("TurnChoiceKind");
    assert(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Pass", (int)CreatureLib::Battling::TurnChoiceKind::Pass);
    assert(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Attack", (int)CreatureLib::Battling::TurnChoiceKind::Attack);
    assert(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Item", (int)CreatureLib::Battling::TurnChoiceKind::Item);
    assert(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Switch", (int)CreatureLib::Battling::TurnChoiceKind::Switch);
    assert(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Flee", (int)CreatureLib::Battling::TurnChoiceKind::Flee);
    assert(r >= 0);
}
void RegisterTurnChoices::RegisterBaseTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("BaseTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("BaseTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::BaseTurnChoice, GetKind), asCALL_THISCALL);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("BaseTurnChoice", "Pokemon@ get_User() const property",
                                     asMETHOD(CreatureLib::Battling::BaseTurnChoice, GetUser), asCALL_THISCALL);
    assert(r >= 0);
}

void RegisterTurnChoices::RegisterMoveTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("MoveTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetKind), asCALL_THISCALL);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "Pokemon@ get_User() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetUser), asCALL_THISCALL);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "LearnedMove@ get_Move() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetAttack), asCALL_THISCALL);
    assert(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "int8 get_Priority() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetPriority), asCALL_THISCALL);
    assert(r >= 0);

    r = engine->RegisterObjectMethod(
        "BaseTurnChoice", "MoveTurnChoice@ opCast()",
        asFUNCTION((refCast<CreatureLib::Battling::BaseTurnChoice, CreatureLib::Battling::AttackTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    assert(r >= 0);
    r = engine->RegisterObjectMethod(
        "MoveTurnChoice", "BaseTurnChoice@ opImplCast()",
        asFUNCTION((refCast<CreatureLib::Battling::AttackTurnChoice, CreatureLib::Battling::BaseTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    assert(r >= 0);
}
